#pragma once

#include <iostream>
#include <string>

namespace string_matcher {

using std::string;

struct IMatcher_impl {
    virtual bool matches(const string&) const = 0;
    virtual void describe(std::ostream&) const = 0;
};

class String_matcher {
public:
    using pimpl_t = std::shared_ptr<IMatcher_impl>;

    template <typename T, typename... Arg>
    static String_matcher make(Arg&&... arg)
    {
        return String_matcher(std::make_shared<T>(std::forward<Arg>(arg)...));
    };

    String_matcher operator&&(const String_matcher&) const;
    String_matcher operator||(const String_matcher&) const;
    String_matcher operator!() const;

    bool matches(const string& subject) const
    { return pimpl_->matches(subject); }

    string to_string() const;

    friend std::ostream&
    operator<<(std::ostream& os, const String_matcher& matcher)
    {
        matcher.pimpl_->describe(os);
        return os;
    }

private:
    String_matcher(pimpl_t pimpl) : pimpl_(pimpl) { }

    pimpl_t pimpl_;
};

String_matcher equals(const string&);
String_matcher substring_at(const string&, size_t);
String_matcher starts_with(const string&);
String_matcher ends_with(const string&);
String_matcher contains(const string&);

}